/*
* l3w_adodbmys is as is. A simple ADO ODBC library (experience) to connect MySql (and much more?) with xBase language
* Copyright (C) 2009 Luigi Ferraris <sviluppi@l3w.it>
* www - http://www.l3w.it
*
* 20090801 - v. 01.00 - first edition: ATTENTION!
*/

/*
*
* Harbour Project source code:
* www - http://www.harbour-project.org
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this software; see the file COPYING.  If not, write to
* the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
* Boston, MA 02111-1307 USA (or visit the web site http://www.gnu.org/).
*
* As a special exception, the Harbour Project gives permission for
* additional uses of the text contained in its release of Harbour.
*
* The exception is that, if you link the Harbour libraries with other
* files to produce an executable, this does not by itself cause the
* resulting executable to be covered by the GNU General Public License.
* Your use of that executable is in no way restricted on account of
* linking the Harbour library code into it.
*
* This exception does not however invalidate any other reasons why
* the executable file might be covered by the GNU General Public License.
*
* This exception applies only to the code released by the Harbour
* Project under the name Harbour.  If you copy code from other
* Harbour Project or Free Software Foundation releases into a copy of
* Harbour, as the General Public License permits, the exception does
* not apply to the code that you add in this way.  To avoid misleading
* anyone as to the status of such modified files, you must delete
* this exception notice from them.
*
* If you write modifications of your own for Harbour, it is your choice
* whether to permit this exception to apply to your modifications.
* If you do not wish that, delete this exception notice.
*/

/* ================================================================================================
*  ADO Connection Section
* See http://www.w3schools.com/ado/ado_ref_connection.asp   &
*
* Attributes  	Sets or returns the attributes of a Connection object
* CommandTimeout 	Sets or returns the number of seconds to wait while attempting to execute a command
* ConnectionString 	Sets or returns the details used to create a connection to a data source
* ConnectionTimeout 	Sets or returns the number of seconds to wait for a connection to open
* CursorLocation 	Sets or returns the location of the cursor service
* DefaultDatabase 	Sets or returns the default database name
* IsolationLevel 	Sets or returns the isolation level
* Mode 	Sets or returns the provider access permission
* Provider 	Sets or returns the provider name
* State 	Returns a value describing if the connection is open or closed
* Version 	Returns the ADO version number
*
* Events
* BeginTransComplete  	Triggered after the BeginTrans operation
* CommitTransComplete 	Triggered after the CommitTrans operation
* ConnectComplete 	Triggered after a connection starts
* Disconnect 	Triggered after a connection ends
* ExecuteComplete 	Triggered after a command has finished executing
* InfoMessage 	Triggered if a warning occurs during a ConnectionEvent operation
* RollbackTransComplete 	Triggered after the RollbackTrans operation
* WillConnect 	Triggered before a connection starts
* WillExecute 	Triggered before a command is executed
*  ================================================================================================ */

/*
* Ado connection Begin Transaction
* Begins a new transaction
* _oConnect = Ado Connection oBject
*/
FUNCTION l3w_AdoConBeTr(_oConnect)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  _oConnect:BeginTrans()
  _lReturned := .T.
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
    MSGSTOP(_oAdoerr:Description,"l3w_AdoConBeTr Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado connection Cancel
* Cancels an execution
* _oConnect = Ado Connection oBject
*/
FUNCTION l3w_AdoConCancel(_oConnect)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oConnect:State != adStateClosed
    IF _oConnect:State != adStateOpen
      _oConnect:Cancel()
    ENDIF
  ENDIF
  _lReturned := .T.
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
    MSGSTOP(_oAdoerr:Description,"l3w_AdoConCancel Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado connection Close
* Closes a connection
* _oConnect = Ado Connection oBject
*/
FUNCTION l3w_AdoConClose(_oConnect)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
l3w_AdoConCancel(_oConnect)
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oConnect:State != adStateClosed
    IF _oConnect:State == adStateOpen
      _oConnect:Close()
    ENDIF
  ENDIF
  _lReturned := .T.
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoConClose Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado connection Commit Transaction
* Saves any changes and ends the current transaction
* _oConnect = Ado Connection oBject
*/
FUNCTION l3w_AdoConComTr(_oConnect)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  _oConnect:CommitTrans()
  _lReturned := .T.
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
    MSGSTOP(_oAdoerr:Description,"l3w_AdoConComTr Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado connection Execute
* Executes a query, statement, procedure or provider specific text
* _oConnect = Ado Connection oBject
*/
FUNCTION l3w_AdoConExecut(_oConnect, cQuery)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
IF VALTYPE(cQuery)=="C" .AND. LEN(ALLTRIM(cQuery))>0
  BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
    _oConnect:Errors:Clear()
    _oConnect:Execute( cQuery )
    _lReturned := .T.
  RECOVER
    FOR EACH _oAdoerr IN _oConnect:Errors
      MSGSTOP(_oAdoerr:Description,"l3w_AdoConExecut Error ")
    NEXT
  END SEQUENCE
ELSE
  MSGSTOP("Parm Error ","l3w_AdoConExecut Error ")
ENDIF
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado connection Open
* Opens a connection
* _oConnect = Ado Connection oBject
* aParms[1] = string - user ID
* aParms[2] = string - user password
* aParms[3] = string - server
* aParms[4] = string - driver
* aParms[5] = string - dsn
* aParms[6] = string - database name
* aParms[7] = string - option see MySql
*/
FUNCTION l3w_AdoConOpen(_oConnect, aParms)
LOCAL sString := "", nJ := 0
LOCAL _lProceed := .T., _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
IF VALTYPE(aParms)<>"A" .AND. LEN(aParms)<>7
  _lProceed := .F.
ENDIF
IF _lProceed == .T.
  sString := "server=" + ALLTRIM(aParms[3]) + ";" + ;
             "driver=" + ALLTRIM(aParms[4]) + ";" + ;
             "dsn=" + ALLTRIM(aParms[5]) + ";" + ;
             "uid=" + ALLTRIM(aParms[1]) + ";" + ;
             "pwd=" + ALLTRIM(aParms[2]) + ";" + ;
             "database=" + ALLTRIM(aParms[6]) + ";" + ;
             "option=" + ALLTRIM(aParms[7])
  BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
    _oConnect:Errors:Clear()
    _oConnect:Open(sString)
    _lReturned := .T.
  RECOVER
    FOR EACH _oAdoerr IN _oConnect:Errors
      MSGSTOP(_oAdoerr:Description,"l3w_AdoConOpen Error ")
    NEXT
  END SEQUENCE
ENDIF
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado connection Open Schema
* Returns schema information from the provider about the data source
* _oConnect = Ado Connection oBject
*/
FUNCTION l3w_AdoConSchema(_oConnect)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  _oConnect:OpenSchema()
  _lReturned := .T.
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
    MSGSTOP(_oAdoerr:Description,"l3w_AdoConSchema Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado connection Roll Back
* Cancels any changes in the current transaction and ends the transaction
* _oConnect = Ado Connection oBject
*/
FUNCTION l3w_AdoConRlBk(_oConnect)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  _oConnect:RollBackTrans()
  _lReturned := .T.
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
    MSGSTOP(_oAdoerr:Description,"l3w_AdoConRlBk Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/* ================================================================================================
*  ADO Recordset Section
* See http://www.w3schools.com/ado/ado_ref_recordset.asp
*
* Event  	Description
* EndOfRecordset 	Triggered when you try to move to a record after the last record
* FetchComplete 	Triggered after all records in an asynchronous operation have been fetched
* FetchProgress 	Triggered periodically in an asynchronous operation, to state how many more records that have been fetched
* FieldChangeComplete 	Triggered after the value of a Field object change
* MoveComplete 	Triggered after the current position in the Recordset has changed
* RecordChangeComplete 	Triggered after a record has changed
* RecordsetChangeComplete 	Triggered after the Recordset has changed
* WillChangeField 	Triggered before the value of a Field object change
* WillChangeRecord 	Triggered before a record change
* WillChangeRecordset 	Triggered before a Recordset change
* WillMove 	Triggered before the current position in the Recordset changes
*
* The Fields Collection's Properties
* Property 	Description
* Count 	Returns the number of items in the fields collection. Starts at zero.
* Example: countfields = rs.Fields.Count
* Item(named_item/number) 	Returns a specified item in the fields collection.
* Example: itemfields = rs.Fields.Item(1) or itemfields = rs.Fields.Item("Name")
*
* The Properties Collection's Properties
* Property 	Description
* Count 	Returns the number of items in the properties collection. Starts at zero.
* Example: countprop = rs.Properties.Count
* Item(named_item/number) 	Returns a specified item in the properties collection.
* Example: itemprop = rs.Properties.Item(1) or itemprop = rs.Properties.Item("Name")
*  ================================================================================================ */

/*
* Ado recordset Add New
* Creates a new record
*/
FUNCTION l3w_AdoRsetAdnew(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:AddNew()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetAdnew Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetAdnew Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Cancel
* Cancels an execution
*/
FUNCTION l3w_AdoRsetCancel(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State != adStateClosed
    _oRset:Cancel()
  ENDIF
  _lReturned := .T.
RECOVER
   FOR EACH _oAdoerr IN _oConnect:Errors
      MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetCancel Error ")
   NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Cancel Batch
* Cancels a batch update
*/
FUNCTION l3w_AdoRsetCabatch(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:CancelBatch()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetCabatch Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetCabatch Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Clone
* Creates a duplicate of an existing Recordset
*/
FUNCTION l3w_AdoRsetClone(_oRset, _oRset2)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset2:= _oRset:Clone()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetClone Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
    MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetClone Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Close
* Closes a Recordset
*/
FUNCTION l3w_AdoRsetClose(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  _lReturned := .T.
  IF _oRset:State != adStateClosed
    _oRset:Close()
  ENDIF
RECOVER
   FOR EACH _oAdoerr IN _oConnect:Errors
      MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetClose Error ")
   NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Compare
* Compares two bookmarks
*/
FUNCTION l3w_AdoRsetCompare(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:CompareBookmarks()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetCompare Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetCompare Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Delete
* Deletes a record or a group of records
*/
FUNCTION l3w_AdoRsetDelete(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:Delete()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetDelete Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetDelete Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Find
* Searches for a record in a Recordset that satisfies a specified criteria
*/
FUNCTION l3w_AdoRsetFind(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:Find()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetFind Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
    MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetFind Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Get Rows
* Copies multiple records from a Recordset object into a two-dimensional array
*/
FUNCTION l3w_AdoRsetGrow(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:GetRows()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetGrow Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
    MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetGrow Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Find
*  Returns a Recordset as a string
*/
FUNCTION l3w_AdoRsetGstring(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:GetString()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetGstring Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
    MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetGstring Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Move
* Moves the record pointer in a Recordset object
*/
FUNCTION l3w_AdoRsetMove(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:Move()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetMove Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
    MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetMove Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Move First
* Moves the record pointer to the first record
*/
FUNCTION l3w_AdoRsetMfirst(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    IF !_oRset:BOF
      _oRset:MoveFirst()
    ENDIF
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetMfirst Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
    MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetMfirst Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Move Last
* Moves the record pointer to the last record
*/
FUNCTION l3w_AdoRsetMlast(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    IF !_oRset:EOF
      _oRset:MoveLast()
    ENDIF
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetMlast Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
    MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetMlast Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Move Next
* Moves the record pointer to the next record
*/
FUNCTION l3w_AdoRsetMnext(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    IF !_oRset:EOF
      _oRset:MoveNext()   // critical da verificare se utile
    ENDIF
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetMnext Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
    MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetMnext Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Move Previous
* Moves the record pointer to the previous record
*/
FUNCTION l3w_AdoRsetMprev(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    IF !_oRset:BOF
      _oRset:MovePrevious()
    ENDIF
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetMprev Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetMprev Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Next Recordset
* Clears the current Recordset object and returns the next Recordset object by looping through a series of commands
*/
FUNCTION l3w_AdoRsetMnrec(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:NextRecordset()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetMnrec Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetMnrec Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Open
* Opens a database element that gives you access to records in a table, the results of a query, or to a saved Recordset
*/
FUNCTION l3w_AdoRsetOpen(_oRset, cString)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
LOCAL _cString := IIF(VALTYPE(cString)=="C" .AND. LEN(ALLTRIM(cString))>0,ALLTRIM(cString),"")
IF LEN(_cString)>0
  l3w_AdoRsetClose(@_oRset)
  BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
    _oConnect:Errors:Clear()
    _oRset:Open(_cString)
    _lReturned := .T.
    RECOVER
    FOR EACH _oAdoerr IN _oConnect:Errors
      MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetOpen Error ")
    NEXT
  END SEQUENCE
ELSE
  MSGSTOP("Manca parametro da eseguire","l3w_AdoRsetOpen Error ")
ENDIF
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Requery
* Updates the data in a Recordset by re-executing the query that made the original Recordset
*/
FUNCTION l3w_AdoRsetRqry(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:Requery()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetRqry Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetRqry Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Resync
* Refreshes the data in the current Recordset from the original database
*/
FUNCTION l3w_AdoRsetRsync(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:Resync()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_doRsetRsync Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetRsync Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Save
* Saves a Recordset object to a file or a Stream object
*/
FUNCTION l3w_AdoRsetSave(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:Save()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetSave Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetSave Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Seek
* Searches the index of a Recordset to find a record that matches the specified values
*/
FUNCTION l3w_AdoRsetSeek(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:Seek()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetSeek Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetSeek Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Supports
* Returns a boolean value that defines whether or not a Recordset object supports a specific type of functionality
*/
FUNCTION l3w_AdoRsetSupp(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:Supports()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetSupp Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetSupp Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Update
* Saves all changes made to a single record  in a Recordset object
*/
FUNCTION l3w_AdoRsetUpdate(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:Update()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetUpdate Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetUpdate Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/*
* Ado recordset Update Batch
* Saves all changes in a Recordset to the database. Used when working in batch update mode
*/
FUNCTION l3w_AdoRsetUpba(_oRset)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    _oRset:UpdateBatch()
    _lReturned := .T.
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetUpba Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetUpba Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

/* ================================================================================================
* da verificare come fare!!! NON usare ancora
*  ADO object Attribute property Section
* See http://www.w3schools.com/ADO/prop_field_attributes.asp#FieldAttributeEnum
* _cProperty = 1 Lunghezza campo -->DefinedSize
* _cProperty = 2 Tipo -->Type
* _cProperty = 3 precisione numerici -->Precision
* _cProperty = 4 cifre dopo la virgola -->NumericScale
*
*  ================================================================================================ */
FUNCTION l3w_AdoFldinfo(_oRset,_cField,_cProperty,_cReturn)
LOCAL _lReturned := .F.
LOCAL _oAdoerr := CreateObject("ADODB.Error")
LOCAL _oConnect := _oRset:ActiveConnection
LOCAL _oField := CreateObject("ADODB.Field")
BEGIN SEQUENCE WITH {|oErr| Break(oErr)}
  _oConnect:Errors:Clear()
  IF _oRset:State == adStateOpen
    FOR EACH _oField IN _oRset:Fields
      IF _oField:Name == _cField
        _lReturned := .T.
        DO CASE
          CASE _cProperty = '1'
            _cReturn := _oField:DefinedSize
          CASE _cProperty = '2'
            _cReturn := _oField:Type
          CASE _cProperty = '3'
            _cReturn := _oField:Precision
          CASE _cProperty = '4'
            _cReturn := _oField:NumericScale
        ENDCASE
      ENDIF
    NEXT
  ELSE
    MSGSTOP("Recordset it's not open","l3w_AdoRsetUpba Error ")
  ENDIF
RECOVER
  FOR EACH _oAdoerr IN _oConnect:Errors
   MSGSTOP(_oAdoerr:Description,"l3w_AdoRsetUpba Error ")
  NEXT
END SEQUENCE
_oAdoerr := NIL
RETURN _lReturned

